/*!
    \file    main.c
    \brief   led spark with systick, USART print and key example

    \version 2024-02-22, V2.1.0, firmware for GD32E23x
*/
// #include "main.h"
#include <stdio.h>
#include "gd32e23x.h"
#include "systick.h"
#include "gd32e23x_libopt.h"
#include "ultrasonic_driver.h"
#include "mlx90614.h"
#include "i2c.h"
#include "led.h"
#include "rs485.h"

extern uint32_t g_capture_value;

uint16_t g_distance_uint16;
uint16_t g_temperature_uint16;

/*!
    \brief      main function
    \param[in]  none
    \param[out] none
    \retval     none
*/
int main(void)
{
    /* configure systick */
    systick_config();


    i2c_gpio_config();
    i2c_config();

    led_config();

    rs485_config();


    /* configure ultrasonic board hardware */
    ultrasonic_gpio_config();
    ultrasonic_receive_config();

    /* ---------- debug start ---------- */

    /* ---------- debug end ---------- */

    while (1)
    {
        delay_ms(ULTRASONIC_TRAN_US);
        ultrasonic_pwm_out_cycles(ULTRASONIC_CYCLES);
        delay_ms(2);
        // printf("cap_val:%ld\t", g_capture_value);
        if (g_capture_value <= CAPTURE_VALUE_MAX) {
            g_distance_uint16 = ultrasonic_calc_distance(g_capture_value);
        } else {
            g_distance_uint16 = 0x0000;
        }
        g_temperature_uint16 = mlx90614_get_objrct_temperature();
    }
}

/* retarget the C library printf function to the USART */
int _write(int fd, char* pBuffer, int size)
{
    for (int i = 0; i < size; i++)
    {
        usart_data_transmit(USART0, (uint8_t)pBuffer[i]);
        while (RESET == usart_flag_get(USART0, USART_FLAG_TBE));
    }
    return size;
}
